/*
 * Copyright (c) 2011, 2012, Endea.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.engine.internal;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import org.sonatype.aether.artifact.Artifact;
import org.sonatype.aether.repository.LocalRepository;
import org.sonatype.aether.repository.RemoteRepository;
import org.sonatype.aether.util.artifact.DefaultArtifact;

import endea.engine.Configuration;

public class ConfigurationImpl implements Configuration {

	private static final String DEFAULT_ARTIFACT = "org.endea:endea.core:[0,)";

	private final Artifact artifact;
	private final String application;
	private final RemoteRepository remoteRepository;
	private final LocalRepository localRepository;

	public ConfigurationImpl(String artifact, String application,
			String remoteRepository, String localRepository) {

		if (artifact == null)
			artifact = DEFAULT_ARTIFACT;
		this.artifact = new DefaultArtifact(artifact);

		this.application = application != null ? application : "endea.Endea$";

		this.remoteRepository = remoteRepository != null ? new RemoteRepository(
				null, "default", remoteRepository) : central_repo;

		this.localRepository = (localRepository != null) ? new LocalRepository(
				localRepository) : new LocalRepository(new File(
				System.getProperty("user.home") + "/.m2/repository"));
	}

	@Override
	public Artifact getArtifact() {
		return artifact;
	}

	@Override
	public String getApplication() {
		return application;
	}

	@Override
	public RemoteRepository getRemoteRepository() {
		return remoteRepository;
	}

	@Override
	public LocalRepository getLocalRepository() {
		return localRepository;
	}

	// //////////////////////////////////////////////////////////////

	private static final String CONF_FILE = "endea.conf";

	private static final String ARTIFACT = "artifact";
	private static final String APPLICATION = "application";
	private static final String REMOTE = "remote";
	private static final String LOCAL = "local";

	private static final String new_line = System.getProperty("line.separator");

	static final RemoteRepository central_repo = new RemoteRepository(
			"central", "default", "http://repo1.maven.org/maven2/");

	public static Configuration parse(String[] tokens) {

		String artifact = null;
		String application = null;
		String remote = null;
		String local = null;

		for (String token : tokens) {
			if (token.contains("//"))
				remote = token.substring(1);
			else if (token.contains(":"))
				artifact = token;
			else if (token.contains("."))
				application = token;
			else
				local = token;
		}

		return new ConfigurationImpl(artifact, application, remote, local);
	}

	public static Configuration load() throws IOException {

		Configuration config;

		File configFile = new File(System.getProperty("user.dir"), CONF_FILE);
		if (!configFile.exists()) {
			config = new ConfigurationImpl(null, null, null, null);
			save(config, configFile);
		} else
			config = load(configFile.toURI().toURL());

		return config;
	}

	static ConfigurationImpl load(URL url) throws IOException {

		String artifact = null;
		String application = null;
		String remote = null;
		String local = null;

		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(url.openStream()));
			String line = null;
			while ((line = reader.readLine()) != null) {

				line = line.trim();
				if (line.isEmpty())
					continue;

				String[] tokens = line.split("\\s+");
				if (tokens.length != 2) {
					System.err.println("Error: " + line);
					continue;
				}
				String name = tokens[0];
				String value = tokens[1];

				if (name.equals(ARTIFACT))
					artifact = value;
				else if (name.equals(APPLICATION))
					application = value;
				else if (name.equals(REMOTE))
					remote = value;
				else if (name.equals(LOCAL))
					local = value;
			}

		} finally {
			close(reader);
		}

		return new ConfigurationImpl(artifact, application, remote, local);
	}

	static void save(Configuration config, File file) throws IOException {

		if (!file.exists()) {
			file.getParentFile().mkdirs();
			file.createNewFile();
		}

		BufferedWriter out = new BufferedWriter(new FileWriter(file));

		try {

			Artifact artifact = config.getArtifact();
			writeAttribute(out, ARTIFACT,
					artifact.getGroupId() + ":" + artifact.getArtifactId()
							+ ":" + artifact.getBaseVersion());
			writeAttribute(out, APPLICATION, config.getApplication());
			writeAttribute(out, REMOTE, config.getRemoteRepository().getUrl());
			writeAttribute(out, LOCAL, config.getLocalRepository().getBasedir()
					.toString());

			out.flush();

		} catch (IOException e) {
			e.printStackTrace(System.err);
			close(out);
		}
	}

	private static void writeAttribute(BufferedWriter out, String name,
			String value) throws IOException {

		out.write(name);
		for (int i = name.length(); i < 12; i++)
			out.write(" ");
		out.write(value);
		out.write(new_line);
	}

	private static void close(Closeable closeable) {

		if (closeable != null)
			try {
				closeable.close();
			} catch (IOException e) {
			}
	}
}