/*
 * Copyright (c) 2011, 2012, Endea.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.engine.internal;

import java.io.PrintStream;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

import org.sonatype.aether.transfer.TransferCancelledException;
import org.sonatype.aether.transfer.TransferEvent;
import org.sonatype.aether.transfer.TransferListener;
import org.sonatype.aether.transfer.TransferResource;

public class ConsoleTransferListener implements TransferListener {

	private static final String new_line = System.getProperty("line.separator");

	private PrintStream out;
	private boolean dot;
	private int dots;

	public ConsoleTransferListener() {
		this(null);
	}

	public ConsoleTransferListener(PrintStream out) {
		this.out = (out != null) ? out : System.out;
	}

	@Override
	public void transferInitiated(TransferEvent event) {

		String message = event.getRequestType() == TransferEvent.RequestType.PUT ? "Uploading"
				: "Downloading";

		beforeWrite();
		out.println(message + ": " + event.getResource().getRepositoryUrl()
				+ event.getResource().getResourceName());
	}

	@Override
	public void transferProgressed(TransferEvent event) {

		dot = true;
		dots++;
		out.print(".");
		if (dots > 60) {
			out.print(new_line);
			dots = 0;
		}
	}

	@Override
	public void transferSucceeded(TransferEvent event) {

		TransferResource resource = event.getResource();
		long contentLength = event.getTransferredBytes();
		if (contentLength >= 0) {
			String type = (event.getRequestType() == TransferEvent.RequestType.PUT ? "Uploaded"
					: "Downloaded");
			String len = contentLength >= 1024 ? toKB(contentLength) + " KB"
					: contentLength + " B";

			String throughput = "";
			long duration = System.currentTimeMillis()
					- resource.getTransferStartTime();
			if (duration > 0) {
				DecimalFormat format = new DecimalFormat("0.0",
						new DecimalFormatSymbols(Locale.ENGLISH));
				double kbPerSec = (contentLength / 1024.0)
						/ (duration / 1000.0);
				throughput = " at " + format.format(kbPerSec) + " KB/sec";
			}

			beforeWrite();
			out.println(type + ": " + resource.getRepositoryUrl()
					+ resource.getResourceName() + " (" + len + throughput
					+ ")");
		}
	}

	@Override
	public void transferFailed(TransferEvent event) {
		beforeWrite();
		out.println(event.getException().getMessage());
		// event.getException().printStackTrace(out);
	}

	@Override
	public void transferCorrupted(TransferEvent event) {
		event.getException().printStackTrace(out);
	}

	@Override
	public void transferStarted(TransferEvent event)
			throws TransferCancelledException {
	}

	private void beforeWrite() {

		if (dot) {
			out.print(new_line);
			dot = false;
			dots = 0;
		}
	}

	private long toKB(long bytes) {
		return (bytes + 1023) / 1024;
	}
}
